// C Source File
// Created 7/19/2002; 6:02:05 PM

/*---------------------------------
		   Arena 3D Level Editor
	      By Malcolm Smith
	      Version v0.5
----------------------------------*/

#define USE_TI89		// Compile for TI-89
#define USE_TI92PLUS		// Compile for TI-92 Plus
#define USE_V200		// Compile for V200

#define OPTIMIZE_ROM_CALLS	// Use ROM Call Optimization

#define MIN_AMS 100		// Compile for AMS 1.00 or higher

#define SAVE_SCREEN		// Save/Restore LCD Contents

#include <tigcclib.h>		// Include All Header Files
#include "extgraph.h"
#include "edit_draw.h"
#include "edit_globals.h"
#include "lvlfind_.h"
#include "levmenu_.h"
#include "keyscan_edit.h"
#include "save.h"


#ifndef FileConstants
#define FileConstants
enum FileConstants
{ HOWMANY = 50, MAXLEN = 9 };
#endif


DEFINE_INT_HANDLER (count_int)
{
  ++count;
}

void
delay (unsigned short del)	//in 55 ms blocks
{
  unsigned short start = count;
  while (count - start < del)	/* nothing */
    ;
}

void
delay1 (unsigned short del)	//in 2.857 ms blocks
{
  unsigned short start = count1;
  while (count1 - start < del)	/* nothing */
    ;
}


char curfolder[9];

short
MakeNewLevel (void)
{
  char tempstr[5];
  clrscr ();
  FontSetSys (F_6x8);
  printf ("\nMap Height [y]:\n");
  InputNum (tempstr, 4);
  map_height = atoi (tempstr);
  if (map_height == 0)
    return 0;

  vidmem = (char *) malloc (sizeof (char) * map_height);
//Loads empty level
  memset (vidmem, 0, map_height);
  FontSetSys (F_6x8);
  printf ("\nOpening empty map...");
  delay (14);
  return 1;
}

//char *gamefolder;
// Main Function
void
_main (void)
{

  unsigned short select_tile[] =	//For XORing as an inverse
  { 0xffff, 0xc003, 0xc003, 0xc003, 0xc003, 0xc003, 0xc003, 0xc003, 
    0xc003, 0xc003, 0xc003, 0xc003, 0xc003, 0xc003, 0xc003, 0xffff
  };
  unsigned short empty_tile[] =	//For clearing out a space for the balls
  { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000
  };



  short pos_y = 0;
  short pos_x = 0;





  char vidmem_cur = 0;
//short error=0;
  short number = 0;
  short key;
  unsigned short count_current;

  char filestr[9] = { 0 };
  
  void *aplane0, *aplane1, *hplane0, *hplane1;
  char buffer[HOWMANY * MAXLEN];
  SYM_ENTRY *symptr = NULL;
  unsigned char *filedata;

  MAP_HEADER cur = { 0, {0}, {0} };

  SaveInt1 = NULL;
  SaveInt5 = NULL;
  screen0 = NULL;
  screen1 = NULL;
  vidmem = NULL;

	//gamefolder = GetGameFolder ();
  //FolderGetCur (curfolder);
  //FolderCur (SYMSTR (gamefolder), TRUE);


  SaveInt1 = GetIntVec (AUTO_INT_1);
  SaveInt5 = GetIntVec (AUTO_INT_5);
  SetIntVec (AUTO_INT_5, count_int);

  ClrScr ();
  FontSetSys (F_6x8);
  DrawStr (0, 10, "Welcome to the", A_NORMAL);
  DrawStr (0, 21, "SkyRacer Level Editor!", A_NORMAL);
  DrawStr (0, 32, "By Malcolm Smith", A_NORMAL);
  DrawStr (0, 50, "89 (92+/V200) Keys...", A_NORMAL);
  DrawStr (0, 60, "Arrows      -Move Around", A_NORMAL);
  DrawStr (0, 70, "2nd (Shift) -Place Ground", A_NORMAL);
  DrawStr (0, 80, "Shift (Z)   -Remove Ground", A_NORMAL);

  key = ngetchx ();
  if (key == KEY_ESC)
    goto leadout;

  ClrScr ();
  FontSetSys (F_8x10);
  DrawStr (2, 2, "Would you like to", A_NORMAL);
  DrawStr (2, 13, "open a map?", A_NORMAL);
  DrawStr (2, 24, "Yes-Enter", A_NORMAL);
  DrawStr (2, 35, "No-Esc", A_NORMAL);


  key = ngetchx ();

  if (key == KEY_ENTER)
    {
      //Load Level
      number = findAllFilesSet (buffer, "srlv", HOWMANY);
 
      if (!number)
	number = -1;
      if (number != -1)		//OK, there are levels on the calc
	{
	  ClrScr ();
	  GKeyFlush();
	  number = domenu (35, 28, buffer, number, 0);
	}

      if (number != -1)		//OK, a level was selected
	{
	  strcpy (filestr, buffer + number * MAXLEN);
	  symptr = DerefSym (SymFind (SYMSTR (filestr)));
	  if (symptr)
	    {
	      filedata = HLock (symptr->handle);
	      filedata += 2;

//next comes a list of MAP_HEADERS
	      MAP_HEADER *header = (MAP_HEADER *) (filedata);
map_height = header->ysize;
strcpy(cur.author,header->author);
strcpy(cur.title,header->title);

//next comes the actual tilemaps
	      unsigned char *map_data =
		(unsigned char *) (filedata +  sizeof (MAP_HEADER));
	      vidmem =
		(char *) malloc (sizeof (char) * map_height);
	      if (vidmem == NULL)
		goto leadout;
	      memcpy (vidmem, map_data,
		      sizeof (char) * map_height);

	    }

	}
      if (number == -1)
	{
	  if (!MakeNewLevel ())
	    goto leadout;
	}

    }
  else				// if (key == KEY_ESC)
    {
      clrscr ();
      if (!MakeNewLevel ())
	goto leadout;
    }




  xmax = X_MAX;
  ymax = (map_height < Y_MAX) ? map_height : Y_MAX;
  pos_y = ymax-1;
  screen_y = map_height - ymax;

//Redirects interrupts to nothing for _rowread
  SetIntVec (AUTO_INT_1, DUMMY_HANDLER);

  screen0 = (char *) malloc (3840);
  screen1 = (char *) malloc (3840);
  if ((screen0 == NULL) || ((screen1 == NULL)))
    goto leadout;
  memset (screen0, 0, 3840);
  memset (screen1, 0, 3840);
  if (!GrayOn ())
    goto leadout;


  keyscan_init ();

  aplane0 = GrayGetPlane (0);
  aplane1 = GrayGetPlane (1);
  hplane0 = screen0;
  hplane1 = screen1;


  keyscan ();


  while (!keystat.escape)
    {

      count_current = count;
      keyscan ();
      draw_edit_level ();

      if (vidmem_cur)
	{
	  GraySprite16_MASK (pos_x * 16, pos_y * 16, 16,
			     GFXTILE (0, (vidmem_cur + 4)), GFXTILE (1,
								     (vidmem_cur
								      + 4)),
			     empty_tile, empty_tile, hplane0, hplane1);
	}
      else
	{
	  GraySprite16_XOR (pos_x * 16, pos_y * 16, 16,
			    select_tile, select_tile, hplane0, hplane1);
	}

      if (keystat.left)
	{
	  pos_x--;
	  if (pos_x == -1)
	    {
	      if (screen_x > 0)
		{
		  screen_x--;
		  pos_x++;
		}
	      else
		pos_x = 0;
	    }
	}
      if (keystat.right)
	{
	  pos_x++;
	  if (pos_x == xmax)
	    {
	      if (screen_x < 8 - xmax)
		{
		  screen_x++;
		  pos_x--;
		}
	      else
		pos_x = xmax - 1;
	    }
	}
      if (keystat.up)
	{
	  pos_y--;
	  if (pos_y == -1)
	    {
	      if (screen_y > 0)
		{
		  pos_y++;
		  screen_y--;
		}
	      else
		pos_y = 0;
	    }
	}
      if (keystat.down)
	{
	  pos_y++;
	  if (pos_y == ymax)
	    {
	      if (screen_y < map_height - ymax)
		{
		  screen_y++;
		  pos_y--;
		}
	      else
		pos_y = ymax - 1;
	    }
	}
      if (keystat.shift)
	{
		EXT_SETPIX(vidmem+pos_y + screen_y,pos_x + screen_x,0);
	}
      if (keystat.second)
	{
		EXT_CLRPIX(vidmem+pos_y + screen_y,pos_x + screen_x,0);
	}
      FastCopyScreen (hplane0, aplane0);
      FastCopyScreen (hplane1, aplane1);
      if (keystat.up || keystat.down || keystat.left || keystat.right
	  || keystat.shift || keystat.second)
	while ((count_current + 4) > count);
    }


  if (GrayCheckRunning ())
    GrayOff ();
  if (screen0 != NULL)
    {
      free (screen0);
      screen0 = NULL;
    }
  if (screen1 != NULL)
    {
      free (screen1);
      screen1 = NULL;
    }
  if (SaveInt1 != NULL)
    SetIntVec (AUTO_INT_1, SaveInt1);
  if (SaveInt5 != NULL)
    SetIntVec (AUTO_INT_5, SaveInt5);
  FontSetSys (F_8x10);
archived:
  clrscr ();
  printf ("Save your map? \nEnter=Yes\n");
  GKeyFlush ();
  key = ngetchx ();

  if (key == KEY_ENTER)
    {
      short flag = 0;
      if (filestr[0] && cur.title[0] && cur.author[0])
	{
	  GKeyFlush ();
	  printf ("\nKeep Same Info?\nEnter=Yes\n");
	  key = ngetchx ();
	  if (key == KEY_ENTER)
	    flag = 1;
	}

      if (!flag)
	{
	  FontSetSys (F_6x8);
	  printf ("Filename:\n");
	  InputStr (filestr, 8);
	}

      if (filestr[0] != 0)
	{			//If a name was typed in
	  if (checkarchived (filestr))
	    {
	      FontSetSys (F_6x8);
	      printf ("\n\nThat level is archived.\n");
	      ngetchx ();
	      FontSetSys (F_8x10);
	      goto archived;
	    }
	  if (!flag)
	    {
	      printf ("\nTitle:\n");
	      InputInfo (cur.title, 24);
	      printf ("\nAuthor:\n");
	      InputInfo (cur.author, 24);
	    }


	  cur.ysize = map_height;

	  savefile (&cur, filestr);
	}


    }

leadout:
  if (GrayCheckRunning ())
    GrayOff ();
  if (symptr)
    HeapUnlock (symptr->handle);
  if (screen0 != NULL)
    {
      free (screen0);
      screen0 = NULL;
    }
  if (screen1 != NULL)
    {
      free (screen1);
      screen1 = NULL;
    }
  if (vidmem != NULL)
    {
      free (vidmem);
      vidmem = NULL;
    }
  //FolderCur (SYMSTR (curfolder), TRUE);
  if (SaveInt1 != NULL)
    SetIntVec (AUTO_INT_1, SaveInt1);
  if (SaveInt5 != NULL)
    SetIntVec (AUTO_INT_5, SaveInt5);
}
